@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -37,10 +32,15 @@
 #include <QDir>
 #include <QDebug>
 #include <QCoreApplication>
+#include <QRegularExpression>
 #include <QStack>
 
 #ifdef Q_OS_WIN
 #include <qt_windows.h>
+#else
+#include <errno.h>
+#include <stdio.h>
+#include <unistd.h>
 #endif
 
 
@@ -98,7 +98,7 @@ static void envExpandWin(QString &args, const Environment *env, const QString *p
         if (prev >= 0) {
             const QString var = args.mid(prev + 1, that - prev - 1).toUpper();
             const QString val = (var == cdName && pwd && !pwd->isEmpty())
-                                    ? QDir::toNativeSeparators(*pwd) : env->value(var);
+                                    ? QDir::toNativeSeparators(*pwd) : env->expandedValueForKey(var);
             if (!val.isEmpty()) { // Empty values are impossible, so this is an existence check
                 args.replace(prev, that - prev + 1, val);
                 off = prev + val.length();
@@ -352,7 +352,7 @@ static QStringList splitArgsUnix(const QString &args, bool abortOnMeta,
                         goto quoteerr;
                     c = args.unicode()[pos++];
                 } while (c != QLatin1Char('\''));
-                cret += args.mid(spos, pos-spos-1);
+                cret += args.midRef(spos, pos - spos - 1);
                 hadWord = true;
             } else if (c == QLatin1Char('"')) {
                 for (;;) {
@@ -398,7 +398,7 @@ static QStringList splitArgsUnix(const QString &args, bool abortOnMeta,
                                 if (abortOnMeta)
                                     goto metaerr; // Assume this is a shell builtin
                             } else {
-                                cret += *vit;
+                                cret += env->expandedValueForKey(env->key(vit));
                             }
                         }
                         if (!braced)
@@ -448,12 +448,12 @@ static QStringList splitArgsUnix(const QString &args, bool abortOnMeta,
                         if (abortOnMeta)
                             goto metaerr; // Assume this is a shell builtin
                     } else {
-                        val = *vit;
+                        val = env->expandedValueForKey(env->key(vit));
                     }
                 }
                 for (int i = 0; i < val.length(); i++) {
-                    QChar cc = val.unicode()[i];
-                    if (cc == 9 || cc == 10 || cc == 32) {
+                    const QChar cc = val.unicode()[i];
+                    if (cc.unicode() == 9 || cc.unicode() == 10 || cc.unicode() == 32) {
                         if (hadWord) {
                             ret += cret;
                             cret.clear();
@@ -539,7 +539,7 @@ QStringList QtcProcess::splitArgs(const QString &args, OsType osType,
 
 QString QtcProcess::quoteArgUnix(const QString &arg)
 {
-    if (!arg.length())
+    if (arg.isEmpty())
         return QString::fromLatin1("''");
 
     QString ret(arg);
@@ -575,7 +575,7 @@ static bool hasSpecialCharsWin(const QString &arg)
 
 static QString quoteArgWin(const QString &arg)
 {
-    if (!arg.length())
+    if (arg.isEmpty())
         return QString::fromLatin1("\"\"");
 
     QString ret(arg);
@@ -583,7 +583,7 @@ static QString quoteArgWin(const QString &arg)
         // Quotes are escaped and their preceding backslashes are doubled.
         // It's impossible to escape anything inside a quoted string on cmd
         // level, so the outer quoting must be "suspended".
-        ret.replace(QRegExp(QLatin1String("(\\\\*)\"")), QLatin1String("\"\\1\\1\\^\"\""));
+        ret.replace(QRegularExpression(QLatin1String("(\\\\*)\"")), QLatin1String("\"\\1\\1\\^\"\""));
         // The argument must not end with a \ since this would be interpreted
         // as escaping the quote -- rather put the \ behind the quote: e.g.
         // rather use "foo"\ than "foo\"
@@ -599,12 +599,12 @@ static QString quoteArgWin(const QString &arg)
 }
 
 QtcProcess::Arguments QtcProcess::prepareArgs(const QString &cmd, SplitError *err, OsType osType,
-                                   const Environment *env, const QString *pwd)
+                                   const Environment *env, const QString *pwd, bool abortOnMeta)
 {
     if (osType == OsTypeWindows)
         return prepareArgsWin(cmd, err, env, pwd);
     else
-        return Arguments::createUnixArgs(splitArgs(cmd, osType, true, err, env, pwd));
+        return Arguments::createUnixArgs(splitArgs(cmd, osType, abortOnMeta, err, env, pwd));
 }
 
 
@@ -626,7 +626,7 @@ void QtcProcess::addArg(QString *args, const QString &arg, OsType osType)
 QString QtcProcess::joinArgs(const QStringList &args, OsType osType)
 {
     QString ret;
-    foreach (const QString &arg, args)
+    for (const QString &arg : args)
         addArg(&ret, arg, osType);
     return ret;
 }
@@ -642,7 +642,7 @@ void QtcProcess::addArgs(QString *args, const QString &inArgs)
 
 void QtcProcess::addArgs(QString *args, const QStringList &inArgs)
 {
-    foreach (const QString &arg, inArgs)
+    for (const QString &arg : inArgs)
         addArg(args, arg);
 }
 
@@ -664,14 +664,22 @@ bool QtcProcess::prepareCommand(const QString &command, const QString &arguments
             if (err != QtcProcess::FoundMeta)
                 return false;
             *outCmd = QLatin1String("/bin/sh");
-            *outArgs = Arguments::createUnixArgs(QStringList()
-                            << QLatin1String("-c")
-                            << (quoteArg(command) + QLatin1Char(' ') + arguments));
+            *outArgs = Arguments::createUnixArgs(
+                        QStringList({"-c", (quoteArg(command) + ' ' + arguments)}));
         }
     }
     return true;
 }
 
+QtcProcess::QtcProcess(QObject *parent)
+    : QProcess(parent)
+{
+    static int qProcessExitStatusMeta = qRegisterMetaType<QProcess::ExitStatus>();
+    static int qProcessProcessErrorMeta = qRegisterMetaType<QProcess::ProcessError>();
+    Q_UNUSED(qProcessExitStatusMeta)
+    Q_UNUSED(qProcessProcessErrorMeta)
+}
+
 void QtcProcess::setUseCtrlCStub(bool enabled)
 {
     // Do not use the stub in debug mode. Activating the stub will shut down
@@ -690,19 +698,11 @@ void QtcProcess::start()
     const OsType osType = HostOsInfo::hostOs();
     if (m_haveEnv) {
         if (m_environment.size() == 0)
-            qWarning("QtcProcess::start: Empty environment set when running '%s'.", qPrintable(m_command));
+            qWarning("QtcProcess::start: Empty environment set when running '%s'.",
+                     qPrintable(m_commandLine.executable().toString()));
         env = m_environment;
 
-        // If the process environment has no libraryPath,
-        // Qt will copy creator's libraryPath into the process environment.
-        // That's brain dead, and we work around it
-        if (osType != OsTypeWindows) {  // a.k.a "Unixoid"
-            const QString libraryPath =
-                QLatin1String(osType == OsTypeMac ? "DYLD_LIBRARY_PATH" : "LD_LIBRARY_PATH");
-            if (env.constFind(libraryPath) == env.constEnd())
-                env.set(libraryPath, QString());
-        }
-        QProcess::setEnvironment(env.toStringList());
+        QProcess::setProcessEnvironment(env.toProcessEnvironment());
     } else {
         env = Environment::systemEnvironment();
     }
@@ -710,13 +710,23 @@ void QtcProcess::start()
     const QString &workDir = workingDirectory();
     QString command;
     QtcProcess::Arguments arguments;
-    bool success = prepareCommand(m_command, m_arguments, &command, &arguments, osType, &env, &workDir);
+    bool success = prepareCommand(m_commandLine.executable().toString(),
+                                  m_commandLine.arguments(),
+                                  &command, &arguments, osType, &env, &workDir);
     if (osType == OsTypeWindows) {
         QString args;
         if (m_useCtrlCStub) {
-            args = QtcProcess::quoteArg(QDir::toNativeSeparators(command));
+            if (m_lowPriority)
+                addArg(&args, "-nice");
+            addArg(&args, QDir::toNativeSeparators(command));
             command = QCoreApplication::applicationDirPath()
                     + QLatin1String("/qtcreator_ctrlc_stub.exe");
+        } else if (m_lowPriority) {
+#ifdef Q_OS_WIN
+            setCreateProcessArgumentsModifier([](CreateProcessArguments *args) {
+                args->flags |= BELOW_NORMAL_PRIORITY_CLASS;
+            });
+#endif
         }
         QtcProcess::addArgs(&args, arguments.toWindowsArgs());
 #ifdef Q_OS_WIN
@@ -730,7 +740,7 @@ void QtcProcess::start()
             setErrorString(tr("Error in command line."));
             // Should be FailedToStart, but we cannot set the process error from the outside,
             // so it would be inconsistent.
-            emit error(QProcess::UnknownError);
+            emit errorOccurred(QProcess::UnknownError);
             return;
         }
         QProcess::start(command, arguments.toUnixArgs());
@@ -1043,7 +1053,7 @@ bool QtcProcess::expandMacros(QString *cmd, AbstractMacroExpander *mx, OsType os
         }
     } else {
         // !Windows
-        MxState state = { MxBasic, false };
+        MxState state = {MxBasic, false};
         QStack<MxState> sstack;
         QStack<MxSave> ostack;
 
@@ -1052,7 +1062,7 @@ bool QtcProcess::expandMacros(QString *cmd, AbstractMacroExpander *mx, OsType os
                 // Our expansion rules trigger in any context
                 if (state.dquote) {
                     // We are within a double-quoted string. Escape relevant meta characters.
-                    rsts.replace(QRegExp(QLatin1String("([$`\"\\\\])")), QLatin1String("\\\\1"));
+                    rsts.replace(QRegularExpression(QLatin1String("([$`\"\\\\])")), QLatin1String("\\\\1"));
                 } else if (state.current == MxSingleQuote) {
                     // We are within a single-quoted string. "Suspend" single-quoting and put a
                     // single escaped quote for each single quote inside the string.
@@ -1089,7 +1099,7 @@ bool QtcProcess::expandMacros(QString *cmd, AbstractMacroExpander *mx, OsType os
                     if (str.unicode()[pos + 1].unicode() == '(') {
                         // $(( starts a math expression. This may also be a $( ( in fact,
                         // so we push the current string and offset on a stack so we can retry.
-                        MxSave sav = { str, pos + 2, varPos };
+                        MxSave sav = {str, pos + 2, varPos};
                         ostack.push(sav);
                         state.current = MxMath;
                         pos += 2;
@@ -1210,6 +1220,19 @@ QString QtcProcess::expandMacros(const QString &str, AbstractMacroExpander *mx,
     return ret;
 }
 
+void QtcProcess::setupChildProcess()
+{
+#if defined Q_OS_UNIX
+    // nice value range is -20 to +19 where -20 is highest, 0 default and +19 is lowest
+    if (m_lowPriority) {
+        errno = 0;
+        if (::nice(5) == -1 && errno != 0)
+            perror("Failed to set nice value");
+    }
+#endif
+    QProcess::setupChildProcess();
+}
+
 bool QtcProcess::ArgIterator::next()
 {
     // We delay the setting of m_prev so we can still delete the last argument
@@ -1319,7 +1342,7 @@ bool QtcProcess::ArgIterator::next()
             }
         }
     } else {
-        MxState state = { MxBasic, false };
+        MxState state = {MxBasic, false};
         QStack<MxState> sstack;
         QStack<int> ostack;
         bool hadWord = false;
