@@ -221,18 +221,14 @@ static void saveWatchers()
 
 static void loadFormats()
 {
-    QVariant value = SessionManager::value("DefaultFormats");
-    QMapIterator<QString, QVariant> it(value.toMap());
-    while (it.hasNext()) {
-        it.next();
+    QMap<QString, QVariant> value = SessionManager::value("DefaultFormats").toMap();
+    for (auto it = value.cbegin(), end = value.cend(); it != end; ++it) {
         if (!it.key().isEmpty())
             theTypeFormats.insert(it.key(), it.value().toInt());
     }
 
-    value = SessionManager::value("IndividualFormats");
-    it = QMapIterator<QString, QVariant>(value.toMap());
-    while (it.hasNext()) {
-        it.next();
+    value = SessionManager::value("IndividualFormats").toMap();
+    for (auto it = value.cbegin(), end = value.cend(); it != end; ++it) {
         if (!it.key().isEmpty())
             theIndividualFormats.insert(it.key(), it.value().toInt());
     }
@@ -241,9 +237,7 @@ static void loadFormats()
 static void saveFormats()
 {
     QMap<QString, QVariant> formats;
-    QHashIterator<QString, int> it(theTypeFormats);
-    while (it.hasNext()) {
-        it.next();
+    for (auto it = theTypeFormats.cbegin(), end = theTypeFormats.cend(); it != end; ++it) {
         const int format = it.value();
         if (format != AutomaticFormat) {
             const QString key = it.key().trimmed();
@@ -254,9 +248,7 @@ static void saveFormats()
     SessionManager::setValue("DefaultFormats", formats);
 
     formats.clear();
-    it = QHashIterator<QString, int>(theIndividualFormats);
-    while (it.hasNext()) {
-        it.next();
+    for (auto it = theIndividualFormats.cbegin(), end = theIndividualFormats.cend(); it != end; ++it) {
         const int format = it.value();
         const QString key = it.key().trimmed();
         if (!key.isEmpty())
@@ -305,7 +297,7 @@ public:
 
     void saveGeometry()
     {
-        SessionManager::setValue("DebuggerSeparateWidgetGeometry", geometry());
+        SessionManager::setValue("DebuggerSeparateWidgetGeometry", QVariant(geometry()));
     }
 
     ~SeparatedView() override
@@ -405,6 +397,7 @@ public:
 class WatchModel : public WatchModelBase
 {
     Q_DECLARE_TR_FUNCTIONS(Debugger::Internal::WatchModel)
+    typedef QSet<WatchItem *> WatchItemSet;
 public:
     WatchModel(WatchHandler *handler, DebuggerEngine *engine);
 
@@ -454,9 +447,13 @@ public:
     void grabWidget();
     void ungrabWidget();
     void timerEvent(QTimerEvent *event) override;
-    int m_grabWidgetTimerId = -1;
+private:
+    QMenu *createFormatMenuForManySelected(const WatchItemSet &item, QWidget *parent);
+    void setItemsFormat(const WatchItemSet &items, const DisplayFormat &format);
+    void addCharsPrintableMenu(QMenu *menu);
 
 public:
+    int m_grabWidgetTimerId = -1;
     WatchHandler *m_handler; // Not owned.
     DebuggerEngine *m_engine; // Not owned.
 
@@ -907,9 +904,9 @@ static QColor valueColor(const WatchItem *item, int column)
                 color = Theme::Debugger_WatchItem_ValueInvalid;
             else if (!model->m_contentsValid && !item->isInspect())
                 color = Theme::Debugger_WatchItem_ValueInvalid;
-            else if (column == 1 && item->value.isEmpty()) // This might still show 0x...
+            else if (item->value.isEmpty()) // This might still show 0x...
                 color = Theme::Debugger_WatchItem_ValueInvalid;
-            else if (column == 1 && item->value != model->m_valueCache.value(item->iname))
+            else if (item->value != model->m_valueCache.value(item->iname))
                 color = Theme::Debugger_WatchItem_ValueChanged;
         }
     }
@@ -1111,7 +1108,8 @@ bool WatchModel::setData(const QModelIndex &idx, const QVariant &value, int role
         }
 
         if (auto kev = ev.as<QKeyEvent>(QEvent::KeyPress)) {
-            if (item && kev->key() == Qt::Key_Delete && item->isWatcher()) {
+            if (item && (kev->key() == Qt::Key_Delete || kev->key() == Qt::Key_Backspace)
+                && item->isWatcher()) {
                 foreach (const QModelIndex &idx, ev.selectedRows())
                     removeWatchItem(itemForIndex(idx));
                 return true;
@@ -1181,7 +1179,7 @@ bool WatchModel::setData(const QModelIndex &idx, const QVariant &value, int role
 Qt::ItemFlags WatchModel::flags(const QModelIndex &idx) const
 {
     if (!idx.isValid())
-        return nullptr;
+        return {};
 
     const WatchItem *item = nonRootItemForIndex(idx);
     if (!item)
@@ -1598,7 +1596,8 @@ static QString removeWatchActionText(QString exp)
 static void copyToClipboard(const QString &clipboardText)
 {
     QClipboard *clipboard = QApplication::clipboard();
-    clipboard->setText(clipboardText, QClipboard::Selection);
+    if (clipboard->supportsSelection())
+        clipboard->setText(clipboardText, QClipboard::Selection);
     clipboard->setText(clipboardText, QClipboard::Clipboard);
 }
 
@@ -1681,29 +1680,37 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
               [this] { grabWidget(); });
 
     menu->addSeparator();
-    menu->addMenu(createFormatMenu(item, menu));
+    QModelIndexList mil = ev.currentOrSelectedRows();
+    if (mil.size() > 1) {
+        WatchItemSet wis;
+        for (const QModelIndex &i : mil)
+            wis.insert(itemForIndex(i));
+
+        menu->addMenu(createFormatMenuForManySelected(wis, menu));
+    } else {
+        menu->addMenu(createFormatMenu(item, menu));
+    }
+
     menu->addMenu(createMemoryMenu(item, menu));
     menu->addMenu(createBreakpointMenu(item, menu));
     menu->addSeparator();
 
-    addAction(menu, tr("Expand All Children"),
-              item,
-              [this, item] {
-                m_expandedINames.insert(item->iname);
-                item->forFirstLevelChildren([this](WatchItem *child) {
-                    m_expandedINames.insert(child->iname);
-                });
-                m_engine->updateLocals();
-              });
+    addAction(menu, tr("Expand All Children"), item, [this, name = item ? item->iname : QString()] {
+        m_expandedINames.insert(name);
+        if (auto item = findItem(name)) {
+            item->forFirstLevelChildren(
+                [this](WatchItem *child) { m_expandedINames.insert(child->iname); });
+            m_engine->updateLocals();
+        }
+    });
 
-    addAction(menu, tr("Collapse All Children"),
-              item,
-              [this, item] {
-                item->forFirstLevelChildren([this](WatchItem *child) {
-                    m_expandedINames.remove(child->iname);
-                });
-                m_engine->updateLocals();
-              });
+    addAction(menu, tr("Collapse All Children"), item, [this, name = item ? item->iname : QString()] {
+        if (auto item = findItem(name)) {
+            item->forFirstLevelChildren(
+                [this](WatchItem *child) { m_expandedINames.remove(child->iname); });
+            m_engine->updateLocals();
+        }
+    });
 
     addAction(menu, tr("Close Editor Tooltips"),
               m_engine->toolTipManager()->hasToolTips(),
@@ -1713,13 +1720,17 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
               true,
               [this] { copyToClipboard(editorContents()); });
 
-    addAction(menu, tr("Copy Current Value to Clipboard"),
+    addAction(menu,
+              tr("Copy Current Value to Clipboard"),
               item,
-              [item] { copyToClipboard(item->value); });
+              [this, name = item ? item->iname : QString()] {
+                  if (auto item = findItem(name))
+                      copyToClipboard(item->value);
+              });
 
-//    addAction(menu, tr("Copy Selected Rows to Clipboard"),
-//              selectionModel()->hasSelection(),
-//              [this] { copyToClipboard(editorContents(selectionModel()->selectedRows())); });
+    //    addAction(menu, tr("Copy Selected Rows to Clipboard"),
+    //              selectionModel()->hasSelection(),
+    //              [this] { copyToClipboard(editorContents(selectionModel()->selectedRows())); });
 
     addAction(menu, tr("Open View Contents in Editor"),
               m_engine->debuggerActionsEnabled(),
@@ -1734,7 +1745,7 @@ bool WatchModel::contextMenuEvent(const ItemViewEvent &ev)
     menu->addAction(action(UseDynamicType));
     menu->addAction(action(SettingsDialog));
 
-    menu->addSeparator();
+    Internal::addHideColumnActions(menu, ev.view());
     menu->addAction(action(SettingsDialog));
     connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
     menu->popup(ev.globalPos());
@@ -1843,6 +1854,20 @@ QMenu *WatchModel::createMemoryMenu(WatchItem *item, QWidget *parent)
     return menu;
 }
 
+void WatchModel::addCharsPrintableMenu(QMenu *menu)
+{
+    auto addBaseChangeAction = [this, menu](const QString &text, int base) {
+        addCheckableAction(menu, text, true, theUnprintableBase == base, [this, base] {
+            theUnprintableBase = base;
+            emit layoutChanged(); // FIXME
+        });
+    };
+    addBaseChangeAction(tr("Treat All Characters as Printable"), 0);
+    addBaseChangeAction(tr("Show Unprintable Characters as Escape Sequences"), -1);
+    addBaseChangeAction(tr("Show Unprintable Characters as Octal"), 8);
+    addBaseChangeAction(tr("Show Unprintable Characters as Hexadecimal"), 16);
+}
+
 QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
 {
     auto menu = new QMenu(tr("Change Value Display Format"), parent);
@@ -1857,17 +1882,7 @@ QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
     const int typeFormat = theTypeFormats.value(stripForFormat(item->type), AutomaticFormat);
     const int individualFormat = theIndividualFormats.value(iname, AutomaticFormat);
 
-    auto addBaseChangeAction = [this, menu](const QString &text, int base) {
-        addCheckableAction(menu, text, true, theUnprintableBase == base, [this, base] {
-            theUnprintableBase = base;
-            emit layoutChanged(); // FIXME
-        });
-    };
-
-    addBaseChangeAction(tr("Treat All Characters as Printable"), 0);
-    addBaseChangeAction(tr("Show Unprintable Characters as Escape Sequences"), -1);
-    addBaseChangeAction(tr("Show Unprintable Characters as Octal"), 8);
-    addBaseChangeAction(tr("Show Unprintable Characters as Hexadecimal"), 16);
+    addCharsPrintableMenu(menu);
 
     const QString spacer = "     ";
     menu->addSeparator();
@@ -1896,12 +1911,18 @@ QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
                            });
     }
 
+    addAction(menu, tr("Reset All Individual Formats"), true, [this]() {
+        theIndividualFormats.clear();
+        saveFormats();
+        m_engine->updateLocals();
+    });
+
     menu->addSeparator();
     addAction(menu, tr("Change Display for Type \"%1\":").arg(item->type), false);
 
     addCheckableAction(menu, spacer + tr("Automatic"), true, typeFormat == AutomaticFormat,
                        [this, item] {
-                            //const QModelIndexList active = activeRows();
+                           //const QModelIndexList active = activeRows();
                            //for (const QModelIndex &idx : active)
                            //    setModelData(LocalsTypeFormatRole, AutomaticFormat, idx);
                            setTypeFormat(item->type, AutomaticFormat);
@@ -1911,11 +1932,76 @@ QMenu *WatchModel::createFormatMenu(WatchItem *item, QWidget *parent)
     for (int format : alternativeFormats) {
         addCheckableAction(menu, spacer + nameForFormat(format), true, format == typeFormat,
                            [this, format, item] {
-                                setTypeFormat(item->type, format);
-                                m_engine->updateLocals();
+                               setTypeFormat(item->type, format);
+                               m_engine->updateLocals();
                            });
     }
 
+    addAction(menu, tr("Reset All Formats for Types"), true, [this]() {
+        theTypeFormats.clear();
+        saveFormats();
+        m_engine->updateLocals();
+    });
+
+    return menu;
+}
+
+void WatchModel::setItemsFormat(const WatchItemSet &items, const DisplayFormat &format)
+{
+    if (format == AutomaticFormat) {
+        for (WatchItem *item : items)
+            theIndividualFormats.remove(item->iname);
+    } else {
+        for (WatchItem *item : items)
+            theIndividualFormats[item->iname] = format;
+    }
+    saveFormats();
+}
+
+QMenu *WatchModel::createFormatMenuForManySelected(const WatchItemSet &items, QWidget *parent)
+{
+    auto menu = new QMenu(tr("Change Display Format for Selected Values"), parent);
+
+    addCharsPrintableMenu(menu);
+
+    QHash<DisplayFormat, int> allItemsFormats;
+    for (WatchItem *item : items) {
+        const DisplayFormats alternativeFormats = typeFormatList(item);
+        for (const DisplayFormat &format : alternativeFormats) {
+            QHash<DisplayFormat, int>::iterator itr = allItemsFormats.find(format);
+            if (itr != allItemsFormats.end())
+                itr.value() += 1;
+            else
+                allItemsFormats[format] = 1;
+        }
+    }
+
+    const QString spacer = "     ";
+    menu->addSeparator();
+
+    addAction(menu, tr("Change Display for Objects"), false);
+    QString msg = QString(tr("Use Display Format Based on Type"));
+    addCheckableAction(menu, spacer + msg, true, false,
+                       [this, items] {
+                            setItemsFormat(items, AutomaticFormat);
+                            m_engine->updateLocals();
+                       });
+
+    int countOfSelectItems = items.size();
+    for (auto it = allItemsFormats.begin(), end = allItemsFormats.end(); it != end; ++it) {
+        DisplayFormat format = it.key();
+        QString formatName = nameForFormat(format);
+        if (formatName.isEmpty())
+            continue;
+
+        addCheckableAction(menu, spacer + formatName,
+                           it.value() == countOfSelectItems,
+                           false,
+                           [this, format, items] {
+                                setItemsFormat(items, format);
+                                m_engine->updateLocals();
+                           });
+    }
     return menu;
 }
 
@@ -2377,7 +2463,7 @@ void WatchModel::clearWatches()
         return;
 
     const QDialogButtonBox::StandardButton ret = CheckableMessageBox::doNotAskAgainQuestion(
-                ICore::mainWindow(), tr("Remove All Expression Evaluators"),
+                ICore::dialogParent(), tr("Remove All Expression Evaluators"),
                 tr("Are you sure you want to remove all expression evaluators?"),
                 ICore::settings(), "RemoveAllWatchers");
     if (ret != QDialogButtonBox::Yes)
@@ -2400,9 +2486,7 @@ QStringList WatchHandler::watchedExpressions()
 {
     // Filter out invalid watchers.
     QStringList watcherNames;
-    QMapIterator<QString, int> it(theWatcherNames);
-    while (it.hasNext()) {
-        it.next();
+    for (auto it = theWatcherNames.cbegin(), end = theWatcherNames.cend(); it != end; ++it) {
         const QString &watcherName = it.key();
         if (!watcherName.isEmpty())
             watcherNames.push_back(watcherName);
@@ -2517,9 +2601,7 @@ QString WatchHandler::typeFormatRequests() const
 {
     QString ba;
     if (!theTypeFormats.isEmpty()) {
-        QHashIterator<QString, int> it(theTypeFormats);
-        while (it.hasNext()) {
-            it.next();
+        for (auto it = theTypeFormats.cbegin(), end = theTypeFormats.cend(); it != end; ++it) {
             const int format = it.value();
             if (format != AutomaticFormat) {
                 ba.append(toHex(it.key()));
@@ -2537,9 +2619,7 @@ QString WatchHandler::individualFormatRequests() const
 {
     QString res;
     if (!theIndividualFormats.isEmpty()) {
-        QHashIterator<QString, int> it(theIndividualFormats);
-        while (it.hasNext()) {
-            it.next();
+        for (auto it = theIndividualFormats.cbegin(), end = theIndividualFormats.cend(); it != end; ++it) {
             const int format = it.value();
             if (format != AutomaticFormat) {
                 res.append(it.key());
@@ -2553,19 +2633,16 @@ QString WatchHandler::individualFormatRequests() const
     return res;
 }
 
-void WatchHandler::appendFormatRequests(DebuggerCommand *cmd)
+void WatchHandler::appendFormatRequests(DebuggerCommand *cmd) const
 {
     QJsonArray expanded;
-    QSetIterator<QString> jt(m_model->m_expandedINames);
-    while (jt.hasNext())
-        expanded.append(jt.next());
+    for (const QString &name : qAsConst(m_model->m_expandedINames))
+        expanded.append(name);
 
     cmd->arg("expanded", expanded);
 
     QJsonObject typeformats;
-    QHashIterator<QString, int> it(theTypeFormats);
-    while (it.hasNext()) {
-        it.next();
+    for (auto it = theTypeFormats.cbegin(), end = theTypeFormats.cend(); it != end; ++it) {
         const int format = it.value();
         if (format != AutomaticFormat)
             typeformats.insert(it.key(), format);
@@ -2573,12 +2650,10 @@ void WatchHandler::appendFormatRequests(DebuggerCommand *cmd)
     cmd->arg("typeformats", typeformats);
 
     QJsonObject formats;
-    QHashIterator<QString, int> it2(theIndividualFormats);
-    while (it2.hasNext()) {
-        it2.next();
-        const int format = it2.value();
+    for (auto it = theIndividualFormats.cbegin(), end = theIndividualFormats.cend(); it != end; ++it) {
+        const int format = it.value();
         if (format != AutomaticFormat)
-            formats.insert(it2.key(), format);
+            formats.insert(it.key(), format);
     }
     cmd->arg("formats", formats);
 }
@@ -2591,18 +2666,16 @@ static inline QJsonObject watcher(const QString &iname, const QString &exp)
     return watcher;
 }
 
-void WatchHandler::appendWatchersAndTooltipRequests(DebuggerCommand *cmd)
+void WatchHandler::appendWatchersAndTooltipRequests(DebuggerCommand *cmd) const
 {
     QJsonArray watchers;
     const DebuggerToolTipContexts toolTips = m_engine->toolTipManager()->pendingTooltips();
     for (const DebuggerToolTipContext &p : toolTips)
         watchers.append(watcher(p.iname, p.expression));
 
-    QMapIterator<QString, int> it(WatchHandler::watcherNames());
-    while (it.hasNext()) {
-        it.next();
+    for (auto it = theWatcherNames.cbegin(), end = theWatcherNames.cend(); it != end; ++it)
         watchers.append(watcher("watch." + QString::number(it.value()), it.key()));
-    }
+
     cmd->arg("watchers", watchers);
 }
 
@@ -2645,10 +2718,6 @@ void WatchHandler::scheduleResetLocation()
     m_model->m_contentsValid = false;
 }
 
-void WatchHandler::resetLocation()
-{
-}
-
 void WatchHandler::setCurrentItem(const QString &iname)
 {
     if (WatchItem *item = m_model->findItem(iname)) {
